@@ -11,16 +11,22 @@
 #include "bwt_lite.h"
 #include "utils.h"
 #include "bwtsw2.h"
-#include "stdaln.h"
 #include "kstring.h"
+#include "bwa.h"
+#include "ksw.h"
 
 #include "kseq.h"
-KSEQ_INIT(gzFile, gzread)
+KSEQ_DECLARE(gzFile)
 
 #include "ksort.h"
 #define __left_lt(a, b) ((a).end > (b).end)
 KSORT_INIT(hit, bsw2hit_t, __left_lt)
 
+#ifdef USE_MALLOC_WRAPPERS
+#  include "malloc_wrap.h"
+#endif
+
+
 extern unsigned char nst_nt4_table[256];
 
 unsigned char nt_comp_table[256] = {
@@ -54,6 +60,8 @@ bsw2opt_t *bsw2_init_opt()
 	o->z = 1; o->is = 3; o->t_seeds = 5; o->hard_clip = 0; o->skip_sw = 0;
 	o->mask_level = 0.50f; o->coef = 5.5f;
 	o->qr = o->q + o->r; o->n_threads = 1; o->chunk_size = 10000000;
+	o->max_chain_gap = 10000;
+	o->cpy_cmt = 0;
 	return o;
 }
 
@@ -91,13 +99,12 @@ bwtsw2_t *bsw2_dup_no_cigar(const bwtsw2_t *b)
 
 void bsw2_extend_left(const bsw2opt_t *opt, bwtsw2_t *b, uint8_t *_query, int lq, uint8_t *pac, bwtint_t l_pac, uint8_t *_mem)
 {
-	int i, matrix[25];
+	int i;
 	bwtint_t k;
 	uint8_t *target = 0, *query;
-	AlnParam par;
+	int8_t mat[25];
 
-	par.matrix = matrix;
-	__gen_ap(par, opt);
+	bwa_fill_scmat(opt->a, opt->b, mat);
 	query = calloc(lq, 1);
 	// sort according to the descending order of query end
 	ks_introsort(hit, b->n, b->hits);
@@ -108,8 +115,7 @@ void bsw2_extend_left(const bsw2opt_t *opt, bwtsw2_t *b, uint8_t *_query, int lq
 	for (i = 0; i < b->n; ++i) {
 		bsw2hit_t *p = b->hits + i;
 		int lt = ((p->beg + 1) / 2 * opt->a + opt->r) / opt->r + lq;
-		int score, j;
-		path_t path;
+		int score, j, qle, tle;
 		p->n_seeds = 1;
 		if (p->l || p->k == 0) continue;
 		for (j = score = 0; j < i; ++j) {
@@ -124,12 +130,12 @@ void bsw2_extend_left(const bsw2opt_t *opt, bwtsw2_t *b, uint8_t *_query, int lq
 		for (k = p->k - 1, j = 0; k > 0 && j < lt; --k) // FIXME: k=0 not considered!
 			target[j++] = pac[k>>2] >> (~k&3)*2 & 0x3;
 		lt = j;
-		score = aln_extend_core(target, lt, query + lq - p->beg, p->beg, &par, &path, 0, p->G, _mem);
+		score = ksw_extend(p->beg, &query[lq - p->beg], lt, target, 5, mat, opt->q, opt->r, opt->bw, 0, -1, p->G, &qle, &tle, 0, 0, 0);
 		if (score > p->G) { // extensible
 			p->G = score;
-			p->len += path.i;
-			p->beg -= path.j;
-			p->k -= path.i;
+			p->k -= tle;
+			p->len += tle;
+			p->beg -= qle;
 		}
 	}
 	free(query); free(target);
@@ -137,62 +143,49 @@ void bsw2_extend_left(const bsw2opt_t *opt, bwtsw2_t *b, uint8_t *_query, int lq
 
 void bsw2_extend_rght(const bsw2opt_t *opt, bwtsw2_t *b, uint8_t *query, int lq, uint8_t *pac, bwtint_t l_pac, uint8_t *_mem)
 {
-	int i, matrix[25];
+	int i;
 	bwtint_t k;
 	uint8_t *target;
-	AlnParam par;
-	
-	par.matrix = matrix;
-	__gen_ap(par, opt);
+	int8_t mat[25];
+
+	bwa_fill_scmat(opt->a, opt->b, mat);
 	target = calloc(((lq + 1) / 2 * opt->a + opt->r) / opt->r + lq, 1);
 	for (i = 0; i < b->n; ++i) {
 		bsw2hit_t *p = b->hits + i;
 		int lt = ((lq - p->beg + 1) / 2 * opt->a + opt->r) / opt->r + lq;
-		int j, score;
-		path_t path;
+		int j, score, qle, tle;
 		if (p->l) continue;
 		for (k = p->k, j = 0; k < p->k + lt && k < l_pac; ++k)
 			target[j++] = pac[k>>2] >> (~k&3)*2 & 0x3;
 		lt = j;
-		score = aln_extend_core(target, lt, query + p->beg, lq - p->beg, &par, &path, 0, 1, _mem);
+		score = ksw_extend(lq - p->beg, &query[p->beg], lt, target, 5, mat, opt->q, opt->r, opt->bw, 0, -1, 1, &qle, &tle, 0, 0, 0) - 1;
 //		if (score < p->G) fprintf(stderr, "[bsw2_extend_hits] %d < %d\n", score, p->G);
 		if (score >= p->G) {
 			p->G = score;
-			p->len = path.i;
-			p->end = path.j + p->beg;
+			p->len = tle;
+			p->end = p->beg + qle;
 		}
 	}
 	free(target);
 }
 
 /* generate CIGAR array(s) in b->cigar[] */
-static void gen_cigar(const bsw2opt_t *opt, int lq, uint8_t *seq[2], const uint8_t *pac, bwtsw2_t *b, const char *name)
+static void gen_cigar(const bsw2opt_t *opt, int lq, uint8_t *seq[2], int64_t l_pac, const uint8_t *pac, bwtsw2_t *b, const char *name)
 {
-	uint8_t *target;
-	int i, matrix[25];
-	AlnParam par;
-	path_t *path;
+	int i;
+	int8_t mat[25];
 
-	par.matrix = matrix;
-	__gen_ap(par, opt);
-	i = ((lq + 1) / 2 * opt->a + opt->r) / opt->r + lq; // maximum possible target length
-	target = calloc(i, 1);
-	path = calloc(i + lq, sizeof(path_t));
-	// generate CIGAR
+	bwa_fill_scmat(opt->a, opt->b, mat);
 	for (i = 0; i < b->n; ++i) {
 		bsw2hit_t *p = b->hits + i;
 		bsw2aux_t *q = b->aux + i;
 		uint8_t *query;
-		bwtint_t k;
-		int score, path_len, beg, end;
+		int beg, end, score;
 		if (p->l) continue;
 		beg = (p->flag & 0x10)? lq - p->end : p->beg;
 		end = (p->flag & 0x10)? lq - p->beg : p->end;
 		query = seq[(p->flag & 0x10)? 1 : 0] + beg;
-		for (k = p->k; k < p->k + p->len; ++k) // in principle, no out-of-boundary here
-			target[k - p->k] = pac[k>>2] >> (~k&3)*2 & 0x3;
-		score = aln_global_core(target, p->len, query, end - beg, &par, path, &path_len);
-		q->cigar = aln_path2cigar32(path, path_len, &q->n_cigar);
+		q->cigar = bwa_gen_cigar(mat, opt->q, opt->r, opt->bw, l_pac, pac, end - beg, query, p->k, p->k + p->len, &score, &q->n_cigar, &q->nm);
 #if 0
 		if (name && score != p->G) { // debugging only
 			int j, glen = 0;
@@ -203,7 +196,7 @@ static void gen_cigar(const bsw2opt_t *opt, int lq, uint8_t *seq[2], const uint8
 					__func__, name, score, p->G, lq, end - beg, p->len, glen, opt->bw);
 		}
 #endif
-		if (beg != 0 || end < lq) { // write soft clipping
+		if (q->cigar && (beg != 0 || end < lq)) { // write soft clipping
 			q->cigar = realloc(q->cigar, 4 * (q->n_cigar + 2));
 			if (beg != 0) {
 				memmove(q->cigar + 1, q->cigar, q->n_cigar * 4);
@@ -216,7 +209,6 @@ static void gen_cigar(const bsw2opt_t *opt, int lq, uint8_t *seq[2], const uint8
 			}
 		}
 	}
-	free(target); free(path);
 }
 
 /* this is for the debugging purpose only */
@@ -227,7 +219,7 @@ void bsw2_debug_hits(const bwtsw2_t *b)
 	for (i = 0; i < b->n; ++i) {
 		bsw2hit_t *p = b->hits + i;
 		if (p->G > 0)
-			printf("G=%d, len=%d, [%d,%d), k=%lu, l=%lu, #seeds=%d, is_rev=%d\n", p->G, p->len, p->beg, p->end, (long)p->k, (long)p->l, p->n_seeds, p->is_rev);
+			printf("G=%d, G2=%d, len=%d, [%d,%d), k=%lu, l=%lu, #seeds=%d, is_rev=%d\n", p->G, p->G2, p->len, p->beg, p->end, (long)p->k, (long)p->l, p->n_seeds, p->is_rev);
 	}
 }
 
@@ -286,12 +278,13 @@ static bwtsw2_t *bsw2_aln1_core(const bsw2opt_t *opt, const bntseq_t *bns, uint8
 		}
 	}
 	b[0] = bb[0][1]; b[1] = bb[1][1]; // bb[*][1] are "narrow SA hits"
-	bsw2_chain_filter(opt, l, b);
+	bsw2_chain_filter(opt, l, b); // NB: only unique seeds are chained
 	for (k = 0; k < 2; ++k) {
 		bsw2_extend_left(opt, bb[k][1], seq[k], l, pac, bns->l_pac, pool->aln_mem);
 		merge_hits(bb[k], l, 0); // bb[k][1] is merged to bb[k][0] here
 		bsw2_resolve_duphits(0, 0, bb[k][0], 0);
 		bsw2_extend_rght(opt, bb[k][0], seq[k], l, pac, bns->l_pac, pool->aln_mem);
+		bsw2_resolve_duphits(0, 0, bb[k][0], 0);
 		b[k] = bb[k][0];
 		free(bb[k]);		
 	}
@@ -403,27 +396,6 @@ static int fix_cigar(const bntseq_t *bns, bsw2hit_t *p, int n_cigar, uint32_t *c
 	return n_cigar;
 }
 
-static int compute_nm(bsw2hit_t *p, int n_cigar, const uint32_t *cigar, const uint8_t *pac, const uint8_t *seq)
-{
-	int k, x, n_mm = 0, i, n_gap = 0;
-	bwtint_t y;
-	x = 0; y = p->k;
-	for (k = 0; k < n_cigar; ++k) {
-		int op  = cigar[k]&0xf;
-		int len = cigar[k]>>4;
-		if (op == 0) { // match
-			for (i = 0; i < len; ++i) {
-				int ref = pac[(y+i)>>2] >> (~(y+i)&3)*2 & 0x3;
-				if (seq[x + i] != ref) ++n_mm;
-			}
-			x += len; y += len;
-		} else if (op == 1) x += len, n_gap += len;
-		else if (op == 2) y += len, n_gap += len;
-		else if (op == 4) x += len;
-	}
-	return n_mm + n_gap;
-}
-
 static void write_aux(const bsw2opt_t *opt, const bntseq_t *bns, int qlen, uint8_t *seq[2], const uint8_t *pac, bwtsw2_t *b, const char *name)
 {
 	int i;
@@ -435,7 +407,7 @@ static void write_aux(const bsw2opt_t *opt, const bntseq_t *bns, int qlen, uint8
 	}
 	b->aux = calloc(b->n, sizeof(bsw2aux_t));
 	// generate CIGAR
-	gen_cigar(opt, qlen, seq, pac, b, name);
+	gen_cigar(opt, qlen, seq, bns->l_pac, pac, b, name);
 	// fix CIGAR, generate mapQ, and write chromosomal position
 	for (i = 0; i < b->n; ++i) {
 		bsw2hit_t *p = &b->hits[i];
@@ -447,8 +419,6 @@ static void write_aux(const bsw2opt_t *opt, const bntseq_t *bns, int qlen, uint8
 			int subo;
 			// fix out-of-boundary CIGAR
 			q->n_cigar = fix_cigar(bns, p, q->n_cigar, q->cigar);
-			// compute the NM tag
-			q->nm = compute_nm(p, q->n_cigar, q->cigar, pac, seq[p->is_rev]);
 			// compute mapQ
 			subo = p->G2 > opt->t? p->G2 : opt->t;
 			if (p->flag>>16 == 1 || p->flag>>16 == 2) c *= .5;
@@ -523,9 +493,10 @@ static void print_hits(const bntseq_t *bns, const bsw2opt_t *opt, bsw2seq1_t *ks
 		bsw2aux_t *q = b->aux + i;
 		int j, beg, end, type = 0;
 		// print mandatory fields before SEQ
+		if (q->cigar == 0) q->flag |= 0x4;
 		ksprintf(&str, "%s\t%d", ks->name, q->flag | (opt->multi_2nd && i? 0x100 : 0));
 		ksprintf(&str, "\t%s\t%ld", q->chr>=0? bns->anns[q->chr].name : "*", (long)q->pos + 1);
-		if (p->l == 0) { // not a repetitive hit
+		if (p->l == 0 && q->cigar) { // not a repetitive hit
 			ksprintf(&str, "\t%d\t", q->pqual);
 			for (k = 0; k < q->n_cigar; ++k)
 				ksprintf(&str, "%d%c", q->cigar[k]>>4, (opt->hard_clip? "MIDNHHP" : "MIDNSHP")[q->cigar[k]&0xf]);
@@ -534,7 +505,7 @@ static void print_hits(const bntseq_t *bns, const bsw2opt_t *opt, bsw2seq1_t *ks
 		else ksprintf(&str, "\t%s\t%d\t%d\t", q->mchr==q->chr? "=" : (q->mchr<0? "*" : bns->anns[q->mchr].name), q->mpos+1, q->isize);
 		// get the sequence begin and end
 		beg = 0; end = ks->l;
-		if (opt->hard_clip) {
+		if (opt->hard_clip && q->cigar) {
 			if ((q->cigar[0]&0xf) == 4) beg += q->cigar[0]>>4;
 			if ((q->cigar[q->n_cigar-1]&0xf) == 4) end -= q->cigar[q->n_cigar-1]>>4;
 		}
@@ -549,7 +520,7 @@ static void print_hits(const bntseq_t *bns, const bsw2opt_t *opt, bsw2seq1_t *ks
 				if (p->flag&0x10) kputc(ks->qual[ks->l - 1 - j], &str);
 				else kputc(ks->qual[j], &str);
 			}
-		} else ksprintf(&str, "\t*");
+		} else kputs("\t*", &str);
 		// print optional tags
 		ksprintf(&str, "\tAS:i:%d\tXS:i:%d\tXF:i:%d\tXE:i:%d\tNM:i:%d", p->G, p->G2, p->flag>>16, p->n_seeds, q->nm);
 		if (q->nn) ksprintf(&str, "\tXN:i:%d", q->nn);
@@ -557,6 +528,12 @@ static void print_hits(const bntseq_t *bns, const bsw2opt_t *opt, bsw2seq1_t *ks
 		if (p->flag&BSW2_FLAG_MATESW) type |= 1;
 		if (p->flag&BSW2_FLAG_TANDEM) type |= 2;
 		if (type) ksprintf(&str, "\tXT:i:%d", type);
+		if (opt->cpy_cmt && ks->comment) {
+			int l = strlen(ks->comment);
+			if (l >= 6 && ks->comment[2] == ':' && ks->comment[4] == ':') {
+				kputc('\t', &str); kputs(ks->comment, &str);
+			}
+		}
 		kputc('\n', &str);
 	}
 	ks->sam = str.s;
@@ -738,41 +715,28 @@ static void process_seqs(bsw2seq_t *_seq, const bsw2opt_t *opt, const bntseq_t *
 	// print and reset
 	for (i = 0; i < _seq->n; ++i) {
 		bsw2seq1_t *p = _seq->seq + i;
-		if (p->sam) printf("%s", p->sam);
+		if (p->sam) err_printf("%s", p->sam);
 		free(p->name); free(p->seq); free(p->qual); free(p->sam);
 		p->tid = -1; p->l = 0;
 		p->name = p->seq = p->qual = p->sam = 0;
 	}
-	fflush(stdout);
+	err_fflush(stdout);
 	_seq->n = 0;
 }
 
-static void kseq_to_bsw2seq(const kseq_t *ks, bsw2seq1_t *p)
-{
-	p->tid = -1;
-	p->l = ks->seq.l;
-	p->name = strdup(ks->name.s);
-	p->seq = strdup(ks->seq.s);
-	p->qual = ks->qual.l? strdup(ks->qual.s) : 0;
-	p->sam = 0;
-}
-
 void bsw2_aln(const bsw2opt_t *opt, const bntseq_t *bns, bwt_t * const target, const char *fn, const char *fn2)
 {
 	gzFile fp, fp2;
 	kseq_t *ks, *ks2;
-	int l, size = 0, is_pe = 0;
+	int l, is_pe = 0, i, n;
 	uint8_t *pac;
 	bsw2seq_t *_seq;
+	bseq1_t *bseq;
 
 	pac = calloc(bns->l_pac/4+1, 1);
-	if (pac == 0) {
-		fprintf(stderr, "[bsw2_aln] insufficient memory!\n");
-		return;
-	}
 	for (l = 0; l < bns->n_seqs; ++l)
-		printf("@SQ\tSN:%s\tLN:%d\n", bns->anns[l].name, bns->anns[l].len);
-	fread(pac, 1, bns->l_pac/4+1, bns->fp_pac);
+		err_printf("@SQ\tSN:%s\tLN:%d\n", bns->anns[l].name, bns->anns[l].len);
+	err_fread_noeof(pac, 1, bns->l_pac/4+1, bns->fp_pac);
 	fp = xzopen(fn, "r");
 	ks = kseq_init(fp);
 	_seq = calloc(1, sizeof(bsw2seq_t));
@@ -781,41 +745,32 @@ void bsw2_aln(const bsw2opt_t *opt, const bntseq_t *bns, bwt_t * const target, c
 		ks2 = kseq_init(fp2);
 		is_pe = 1;
 	} else fp2 = 0, ks2 = 0, is_pe = 0;
-	while (kseq_read(ks) >= 0) {
-		if (ks->name.l > 2 && ks->name.s[ks->name.l-2] == '/')
-			ks->name.l -= 2, ks->name.s[ks->name.l] = 0;
-		if (_seq->n == _seq->max) {
-			_seq->max = _seq->max? _seq->max<<1 : 1024;
+	while ((bseq = bseq_read(opt->chunk_size * opt->n_threads, &n, ks, ks2)) != 0) {
+		int size = 0;
+		if (n > _seq->max) {
+			_seq->max = n;
+			kroundup32(_seq->max);
 			_seq->seq = realloc(_seq->seq, _seq->max * sizeof(bsw2seq1_t));
 		}
-		kseq_to_bsw2seq(ks, &_seq->seq[_seq->n++]);
-		size += ks->seq.l;
-		if (ks2) {
-			if (kseq_read(ks2) >= 0) {
-				if (ks2->name.l > 2 && ks2->name.s[ks2->name.l-2] == '/')
-					ks2->name.l -= 2, ks2->name.s[ks2->name.l] = 0;
-				kseq_to_bsw2seq(ks2, &_seq->seq[_seq->n++]); // for PE, _seq->n here must be odd and we do not need to enlarge
-				size += ks->seq.l;
-			} else {
-				fprintf(stderr, "[%s] The second query file has fewer reads. Switched to the single-end mode for the following batches.\n", __func__);
-				is_pe = 0;
-			}
-		}
-		if (size > opt->chunk_size * opt->n_threads) {
-			fprintf(stderr, "[bsw2_aln] read %d sequences/pairs (%d bp)...\n", _seq->n, size);
-			process_seqs(_seq, opt, bns, pac, target, is_pe);
-			size = 0;
+		_seq->n = n;
+		for (i = 0; i < n; ++i) {
+			bseq1_t *b = &bseq[i];
+			bsw2seq1_t *p = &_seq->seq[i];
+			p->tid = -1; p->l = b->l_seq;
+			p->name = b->name; p->seq = b->seq; p->qual = b->qual; p->comment = b->comment; p->sam = 0;
+			size += p->l;
 		}
+		fprintf(stderr, "[bsw2_aln] read %d sequences/pairs (%d bp) ...\n", n, size);
+		free(bseq);
+		process_seqs(_seq, opt, bns, pac, target, is_pe);
 	}
-	fprintf(stderr, "[bsw2_aln] read %d sequences/pairs (%d bp)...\n", _seq->n, size);
-	process_seqs(_seq, opt, bns, pac, target, is_pe);
 	// free
 	free(pac);
 	free(_seq->seq); free(_seq);
 	kseq_destroy(ks);
-	gzclose(fp);
+	err_gzclose(fp);
 	if (fn2) {
 		kseq_destroy(ks2);
-		gzclose(fp2);
+		err_gzclose(fp2);
 	}
 }
