@@ -2,14 +2,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "utils.h"
 #include "bwt.h"
 #include "bntseq.h"
 #include "bwtsw2.h"
 #include "kstring.h"
-#ifndef _NO_SSE2
 #include "ksw.h"
-#else
-#include "stdaln.h"
+
+#ifdef USE_MALLOC_WRAPPERS
+#  include "malloc_wrap.h"
 #endif
 
 #define MIN_RATIO     0.8
@@ -24,7 +25,6 @@ typedef struct {
 
 bsw2pestat_t bsw2_stat(int n, bwtsw2_t **buf, kstring_t *msg, int max_ins)
 {
-	extern void ks_introsort_uint64_t(size_t n, uint64_t *a);
 	int i, k, x, p25, p50, p75, tmp, max_len = 0;
 	uint64_t *isize;
 	bsw2pestat_t r;
@@ -44,13 +44,13 @@ bsw2pestat_t bsw2_stat(int n, bwtsw2_t **buf, kstring_t *msg, int max_ins)
 		max_len = max_len > t[1]->end - t[1]->beg? max_len : t[1]->end - t[1]->beg;
 		isize[k++] = l;
 	}
-	ks_introsort_uint64_t(k, isize);
+	ks_introsort_64(k, isize);
 	p25 = isize[(int)(.25 * k + .499)];
 	p50 = isize[(int)(.50 * k + .499)];
 	p75 = isize[(int)(.75 * k + .499)];
 	ksprintf(msg, "[%s] infer the insert size distribution from %d high-quality pairs.\n", __func__, k);
 	if (k < 8) {
-		ksprintf(msg, "[%s] fail to infer the insert size distribution.\n", __func__);
+		ksprintf(msg, "[%s] fail to infer the insert size distribution: too few good pairs.\n", __func__);
 		free(isize);
 		r.failed = 1;
 		return r;
@@ -59,6 +59,12 @@ bsw2pestat_t bsw2_stat(int n, bwtsw2_t **buf, kstring_t *msg, int max_ins)
 	r.low  = tmp > max_len? tmp : max_len;
 	if (r.low < 1) r.low = 1;
 	r.high = (int)(p75 + OUTLIER_BOUND * (p75 - p25) + .499);
+	if (r.low > r.high) {
+		ksprintf(msg, "[%s] fail to infer the insert size distribution: upper bound is smaller than max read length.\n", __func__);
+		free(isize);
+		r.failed = 1;
+		return r;
+	}
 	ksprintf(msg, "[%s] (25, 50, 75) percentile: (%d, %d, %d)\n", __func__, p25, p50, p75);
 	ksprintf(msg, "[%s] low and high boundaries for computing mean and std.dev: (%d, %d)\n", __func__, r.low, r.high);
 	for (i = x = 0, r.avg = 0; i < k; ++i)
@@ -74,9 +80,9 @@ bsw2pestat_t bsw2_stat(int n, bwtsw2_t **buf, kstring_t *msg, int max_ins)
 	r.low  = tmp > max_len? tmp : max_len;
 	if (r.low < 1) r.low = 1;
 	r.high = (int)(p75 + 3. * (p75 - p25) + .499);
-	if (r.low > r.avg - MAX_STDDEV * 4.) r.low = (int)(r.avg - MAX_STDDEV * 4. + .499);
+	if (r.low > r.avg - MAX_STDDEV * r.std) r.low = (int)(r.avg - MAX_STDDEV * r.std + .499);
 	r.low = tmp > max_len? tmp : max_len;
-	if (r.high < r.avg - MAX_STDDEV * 4.) r.high = (int)(r.avg + MAX_STDDEV * 4. + .499);
+	if (r.high < r.avg - MAX_STDDEV * r.std) r.high = (int)(r.avg + MAX_STDDEV * r.std + .499);
 	ksprintf(msg, "[%s] low and high boundaries for proper pairs: (%d, %d)\n", __func__, r.low, r.high);
 	free(isize);
 	return r;
@@ -126,54 +132,25 @@ void bsw2_pair1(const bsw2opt_t *opt, int64_t l_pac, const uint8_t *pac, const b
 		for (i = 0; i < l_mseq; ++i) // on the forward strand
 			seq[i] = nst_nt4_table[(int)mseq[i]];
 	}
-#ifndef _NO_SSE2
 	{
-		ksw_query_t *q;
-		ksw_aux_t aux[2];
-		// forward Smith-Waterman
-		aux[0].T = opt->t; aux[0].gapo = opt->q; aux[0].gape = opt->r; aux[1] = aux[0];
-		q = ksw_qinit(l_mseq * g_mat[0] < 250? 1 : 2, l_mseq, seq, 5, g_mat);
-		ksw_sse2(q, end - beg, ref, &aux[0]);
-		free(q);
-		if (aux[0].score < opt->t) {
-			free(seq);
-			return;
-		}
-		++aux[0].qe; ++aux[0].te;
-		// reverse Smith-Waterman
-		seq_reverse(aux[0].qe, seq, 0);
-		seq_reverse(aux[0].te, ref, 0);
-		q = ksw_qinit(aux[0].qe * g_mat[0] < 250? 1 : 2, aux[0].qe, seq, 5, g_mat);
-		ksw_sse2(q, aux[0].te, ref, &aux[1]);
-		free(q);
-		++aux[1].qe; ++aux[1].te;
-		// write output
-		a->G = aux[0].score;
-		a->G2 = aux[0].score2 > aux[1].score2? aux[0].score2 : aux[1].score2;
-		if (a->G2 < opt->t) a->G2 = 0;
-		if (a->G2) a->flag |= BSW2_FLAG_TANDEM;
-		a->k = beg + (aux[0].te - aux[1].te);
-		a->len = aux[1].te;
-		a->beg = aux[0].qe - aux[1].qe;
-		a->end = aux[0].qe;
-	}
-#else
-	{
-		AlnParam ap;
-		path_t path[2];
-		int matrix[25];
-		for (i = 0; i < 25; ++i) matrix[i] = g_mat[i];
-		ap.gap_open = opt->q; ap.gap_ext = opt->r; ap.gap_end = opt->r;
-		ap.matrix = matrix; ap.row = 5; ap.band_width = 50;
-		a->G = aln_local_core(ref, end - beg, seq, l_mseq, &ap, path, 0, opt->t, &a->G2);
+		int flag = KSW_XSUBO | KSW_XSTART | (l_mseq * g_mat[0] < 250? KSW_XBYTE : 0) | opt->t;
+		kswr_t aln;
+		aln = ksw_align(l_mseq, seq, end - beg, ref, 5, g_mat, opt->q, opt->r, flag, 0);
+		a->G = aln.score;
+		a->G2 = aln.score2;
 		if (a->G < opt->t) a->G = 0;
 		if (a->G2 < opt->t) a->G2 = 0;
-		a->k = beg + path[0].i - 1;
-		a->len = path[1].i - path[0].i + 1;
-		a->beg = path[0].j - 1;
-		a->end = path[1].j;
+		if (a->G2) a->flag |= BSW2_FLAG_TANDEM;
+		a->k = beg + aln.tb;
+		a->len = aln.te - aln.tb + 1;
+		a->beg = aln.qb;
+		a->end = aln.qe + 1;
+		/*
+		printf("[Q] "); for (i = 0; i < l_mseq; ++i) putchar("ACGTN"[(int)seq[i]]); putchar('\n');
+		printf("[R] "); for (i = 0; i < end - beg; ++i) putchar("ACGTN"[(int)ref[i]]); putchar('\n');
+		printf("G=%d,G2=%d,beg=%d,end=%d,k=%lld,len=%d\n", a->G, a->G2, a->beg, a->end, a->k, a->len);
+		*/
 	}
-#endif
 	if (a->is_rev) i = a->beg, a->beg = l_mseq - a->end, a->end = l_mseq - i;
 	free(seq);
 }
